##
##
##    wing_modularity.R
##
##    A script for analysis of morphological and evolutionary modularity
##    wing shape data of scanned scanned bird wings
##
##    For Rader and Hedrick 2021 submission
##
##    Original version: 2021, Jonathan Rader
##
################################################################################

## Set working directory -------------------------------------------------------
setwd() #Your WD here

## First, we should do some housekeeping ---------------------------------------
require(dispRity) # for morphological disparity analysis
require(dplyr)    # for data  wrangling
require(rddtools) # for regression discontinuity estimators
require(phytools)       # for tree manipulation and phylogenetic AOV
source("Data and Scripts/modTest.R") # modularity test function

## Read in the data and phylogenetic tree ---------------------------
camber <- read.csv("Data and Scripts/camber.csv", header=TRUE)
chord <- read.csv("Data and Scripts/chord.csv", header=TRUE)
xst <- read.csv("Data and Scripts/xst.csv", header=TRUE)
xsa <- read.csv("Data and Scripts/xsa.csv", header=TRUE)
sp.summary <- read.csv("Data and Scripts/species_summary.csv", header=TRUE)

smalltree <- read.tree("Data and Scripts/smalltree.tre")
plotTree(smalltree, type="fan") # plot to check that it worked correctly
dev.off()

## Now match up tree to tip data for wingDat data -------------------------------------------------------------------
# Order data to match tree order
setdiff(sp.summary$Species, smalltree$tip.label) # First, make sure that all of the data are in the tree
# Phalaropus_tricolor isn't, so we'll remove it from the data
sp.summary <- droplevels(subset(sp.summary, sp.summary$Species != "Phalaropus_tricolor"))
sp.summary <- sp.summary[order(sp.summary$Species),][rank(smalltree$tip.label),]

## Build some data matrices for each of the variables --------------------------
slicerange <- 5:39 # select the columns from the datasets for analysis

camber.mat <- data.frame(chord[,1], camber[,slicerange]) # grab the slices from camber
colnames(camber.mat)[1] <- "Species"
camber.mat.log <- data.frame(chord[,1], log(camber[,slicerange],10))
colnames(camber.mat.log)[1] <- "Species"
camber.summary <- ddply(camber.mat, .(Species), colwise(median), na.action="omit")
row.names(camber.summary) <- camber.summary$Species
# Now let's calculate SE
camber.sd <- ddply(camber.mat.log, .(Species), colwise(sd, na.rm=TRUE))
camber.n <- camber.mat.log %>%
  group_by(Species) %>%
  summarise(count = length(Species))
camber.se <- data.frame(mapply(`/`, data.frame(camber.sd[,-1]), camber.n[,2]))
# The species Phalaropus tricolor isn't represented in our tree, so we'll remove it
camber.summary <- droplevels(subset(camber.summary, camber.summary$Species != "Phalaropus_tricolor"))
# Align our dataframes with the tree tips
camber.summary <- camber.summary[order(camber.summary$Species),][rank(sp.summary$Species),]
# Prune out missing data that won't play well in subsequent analyses
camber.summary <- camber.summary[rowSums(is.na(camber.summary)) == 0,]
camber.summary <- camber.summary[,2:length(colnames(camber.summary))]
camber.summary.log <- log(camber.summary,10)

chord.mat <- data.frame(chord[,1], chord[,slicerange]) # grab the slices from chord
colnames(chord.mat)[1] <- "Species"
chord.mat.log <- data.frame(chord[,1], log(chord[,slicerange],10))
colnames(chord.mat.log)[1] <- "Species"
chord.summary <- ddply(chord.mat, .(Species), colwise(median), na.action="omit")
row.names(chord.summary) <- chord.summary$Species
# Now let's calculate SE
chord.sd <- ddply(chord.mat.log, .(Species), colwise(sd, na.rm=TRUE))
chord.n <- chord.mat.log %>%
  group_by(Species) %>%
  summarise(count = length(Species))
chord.se <- data.frame(mapply(`/`, data.frame(chord.sd[,-1]), chord.n[,2]))
chord.summary <- droplevels(subset(chord.summary, chord.summary$Species != "Phalaropus_tricolor"))
chord.summary <- chord.summary[order(chord.summary$Species),][rank(sp.summary$Species),]
chord.summary <- chord.summary[rowSums(is.na(chord.summary)) == 0,]
# non-dimensionlize the data
chord.summary.nd <- chord.summary[,2:length(colnames(chord.summary))]/sp.summary$massthird
chord.summary.log <- log(chord.summary.nd,10)

xsa.mat <- data.frame(chord[,1], xsa[,slicerange]) # grab the slices from XSA
colnames(xsa.mat)[1] <- "Species"
xsa.mat.log <- data.frame(xsa[,1], log(xsa[,slicerange],10))
colnames(xsa.mat.log)[1] <- "Species"
xsa.summary <- ddply(xsa.mat, .(Species), colwise(median), na.action="omit")
row.names(xsa.summary) <- xsa.summary$Species
# Now let's calculate SE
xsa.sd <- ddply(xsa.mat.log, .(Species), colwise(sd, na.rm=TRUE))
xsa.n <- xsa.mat.log %>%
  group_by(Species) %>%
  summarise(count = length(Species))
xsa.se <- data.frame(mapply(`/`, data.frame(xsa.sd[,-1]), xsa.n[,2]))
rownames(xsa.se) <- xsa.sd$Species
xsa.summary <- droplevels(subset(xsa.summary, xsa.summary$Species != "Phalaropus_tricolor"))
xsa.summary <- xsa.summary[order(xsa.summary$Species),][rank(sp.summary$Species),]
xsa.summary <- xsa.summary[rowSums(is.na(xsa.summary)) == 0,]
xsa.summary.nd <- xsa.summary[,2:length(colnames(xsa.summary))]/sp.summary$masstwothird
xsa.summary.log <- log(xsa.summary.nd,10)


xst.mat <- data.frame(chord[,1], xst[,slicerange]) # grab the slices from XST
colnames(xst.mat)[1] <- "Species"
xst.mat.log <- data.frame(xst[,1], log(xst[,slicerange],10))
colnames(xst.mat.log)[1] <- "Species"
xst.summary <- ddply(xst.mat, .(Species), colwise(median), na.action="omit")
row.names(xst.summary) <- xst.summary$Species
# Now let's calculate SE
xst.sd <- ddply(xst.mat.log, .(Species), colwise(sd, na.rm=TRUE))
xst.n <- xst.mat.log %>%
  group_by(Species) %>%
  summarise(count = length(Species))
xst.se <- data.frame(mapply(`/`, data.frame(xst.sd[,-1]), xst.n[,2]))
rownames(xst.se) <- xst.sd$Species
xst.summary <- droplevels(subset(xst.summary, xst.summary$Species != "Phalaropus_tricolor"))
xst.summary <- xst.summary[order(xst.summary$Species),][rank(sp.summary$Species),]
xst.summary <- xst.summary[rowSums(is.na(xst.summary)) == 0,]
xst.summary.nd <- xst.summary[,2:length(colnames(xst.summary))]/sp.summary$massthird
xst.summary.log <- log(xst.summary.nd,10)

## Some basic summary stats ----------------------------------------------------
# Camber
hw.camb.mean <- rowMeans(camber.summary[,1:19])
median(hw.camb.mean)  # 0.07284211
mad(hw.camb.mean)  # 0.01747907
aw.camb.mean <- rowMeans(camber.summary[,24:35])
median(aw.camb.mean)  # 0.111875
mad(aw.camb.mean)  # 0.02143593

# Chord
hw.chord.mean <- rowMeans(chord.summary.nd[,1:19])
median(hw.chord.mean)  # 13.11567
mad(hw.chord.mean)  # 2.531688
aw.chord.mean <- rowMeans(chord.summary.nd[,24:35])
median(aw.chord.mean)  # 20.42722
mad(aw.chord.mean)  # 2.591819

## Set up grouping -------------------------------------------------------------
# Set ranges of wing slices that represent each wing region
hwidx <- c(min(slicerange):25)
awidx <- c(26:max(slicerange))
partitionidx <- vector(mode="character", length=length(slicerange)) # not sure if this is doing anything
partitionidx[1:length(hwidx)] <- "hw"
partitionidx[(length(hwidx)+1):length(partitionidx)] <- "aw"

partition.wrist <- c(rep(1,17), rep(2,5), rep(3,13))
partition.nowrist <- c(rep(1,17), rep(2,13))

## Run the modularity tests ----------------------------------------------------
iters <- 10000 # Set the number of bootstrap iterations

# Run the test on each individual variable
modtest.camber <- modularity.test(camber.summary.log, partitionidx, iters)
modtest.chord <- modularity.test(chord.summary.log, partitionidx, iters)
modtest.xsa <- modularity.test(xsa.summary.log, partitionidx, iters)
modtest.xst <- modularity.test(xst.summary.log, partitionidx, iters)

# What happens if we drop the wrist entirely?
# using log-transformed data
camber.w <- camber.summary.log[,which(partition.wrist != 2)]
chord.w <- chord.summary.log[,which(partition.wrist != 2)]
xsa.w <- xsa.summary.log[,which(partition.wrist != 2)]
xst.w <- xst.summary.log[,which(partition.wrist != 2)]

modtest.camber.nw <- modularity.test(camber.w, partition.nowrist, iters)
modtest.chord.nw <- modularity.test(chord.w, partition.nowrist, iters)
modtest.xsa.nw <- modularity.test(xsa.w, partition.nowrist, iters)
modtest.xst.nw <- modularity.test(xst.w, partition.nowrist, iters)

## Now we'll work on morphological disparity -----------------------------------
# camb.disparity <- dispRity(camber.summary, metric = c(variances), tree = smalltree)
# chord.disparity <- dispRity(chord.summary.nd, metric = c(variances), tree = smalltree)
# xsa.disparity <- dispRity(xsa.summary.nd, metric = c(variances), tree = smalltree)
# xst.disparity <- dispRity(xst.summary.nd, metric = c(variances), tree = smalltree)

# Run disparity analysis on the log-transformed data
camb.disparity.l <- dispRity(camber.summary.log, metric = c(variances), tree = smalltree)
chord.disparity.l <- dispRity(chord.summary.log, metric = c(variances), tree = smalltree)
xsa.disparity.l <- dispRity(xsa.summary.log, metric = c(variances), tree = smalltree)
xst.disparity.l <- dispRity(xst.summary.log, metric = c(variances), tree = smalltree)

## And now evolutionary rate ---------------------------------------------------
iter=1000
modlist <- c("BM", "OU", "EB")

camb.Sig2 <- data.frame()
camb.aicc <- data.frame()
for(j in 1:length(modlist)){
  for(i in 1:length(camber.summary.log[1,])){
    # Incorporate SE
    foo <- fitContinuous(smalltree, setNames(camber.summary.log[,i], rownames(camber.summary.log)), model= modlist[j], 
                         SE = setNames(camber.se[,i], rownames(camber.summary.log)), control=list(niter=iter))
    camb.Sig2[j,i] <- foo$opt$sigsq
    camb.aicc[j,i] <- foo$opt$aicc
  }
}
cambdelaic <- matrix(nrow=length(modlist), ncol=length(slicerange))
min.aicc <- apply(camb.aicc,2,min)
for(i in 1:length(slicerange)){
  foo <- (camb.aicc[[i]] - min.aicc[i])
  cambdelaic[,i] <- as.numeric(foo)
}
min(cambdelaic[cambdelaic !=0])  # 11.325

# row.names(camb.aicc) <- modlist
# rowMeans(camb.aicc)
# BM        OU        EB        delta 
# -365.2132 -392.7888 -363.8530 -383.3615 

#matplot(t(camb.Sig2),type="l", lty=1, xlab="Camber")

chord.Sig2 <- data.frame()
chord.aicc <- data.frame()
for(j in 1:length(modlist)){
  for(i in 1:length(chord.summary.log[1,])){
    # Incorporate SE
    foo <- fitContinuous(smalltree, setNames(chord.summary.log[,i], rownames(chord.summary.log)), model= modlist[j], 
                         SE = setNames(chord.se[,i], rownames(chord.summary.log)), control=list(niter=iter))
    chord.Sig2[j,i] <- foo$opt$sigsq
    chord.aicc[j,i] <- foo$opt$aicc
  }
}
chorddelaic <- matrix(nrow=length(modlist), ncol=length(slicerange))
min.aicc <- apply(chord.aicc,2,min)
for(i in 1:length(slicerange)){
  foo <- chord.aicc[[i]] - min.aicc[i]
  chorddelaic[,i] <- as.numeric(foo)
}
min(chorddelaic[chorddelaic !=0])  # 4.80665

# row.names(chord.aicc) <- modlist
# rowMeans(chord.aicc) 
# BM        OU        EB        delta 
# -452.8506 -463.6128 -450.4348 -459.2038 

#matplot(t(chord.Sig2),type="l", lty=1, xlab="Chord")

xsa.Sig2 <- data.frame()
xsa.aicc <- data.frame()
for(j in 1:length(modlist)){
  for(i in 1:length(xsa.summary.log[1,])){
    # Incorporate SE
    foo <- fitContinuous(smalltree, setNames(xsa.summary.log[,i], rownames(xsa.summary.log)), model= modlist[j], 
                         SE = setNames(xsa.se[,i], rownames(xsa.summary.log)), control=list(niter=iter))
    xsa.Sig2[j,i] <- foo$opt$sigsq
    xsa.aicc[j,i] <- foo$opt$aicc
  }
}
xsadelaic <- matrix(nrow=length(modlist), ncol=length(slicerange))
min.aicc <- apply(xsa.aicc,2,min)
for(i in 1:length(slicerange)){
  foo <- xsa.aicc[[i]] - min.aicc[i]
  xsadelaic[,i] <- as.numeric(foo)
}
min(xsadelaic[xsadelaic !=0])  # 5.905596

# row.names(xsa.aicc) <- modlist
# rowMeans(xsa.aicc)
# BM        OU        EB        delta 
# -172.8935 -184.1546 -169.7849 -181.8823 

#matplot(t(xsa.Sig2),type="l", lty=1, xlab="XSA")

xst.Sig2 <- data.frame()
xst.aicc <- data.frame()
for(j in 1:length(modlist)){
  for(i in 1:length(xst.summary.log[1,])){
    # Incorporate SE
    foo <- fitContinuous(smalltree, setNames(xst.summary.log[,i], rownames(xst.summary.log)), model= modlist[j], 
                         SE = setNames(xst.se[,i], rownames(xst.summary.log)), control=list(niter=iter))
    xst.Sig2[j,i] <- foo$opt$sigsq
    xst.aicc[j,i] <- foo$opt$aicc
  }
}
xstdelaic <- matrix(nrow=length(modlist), ncol=length(slicerange))
min.aicc <- apply(xst.aicc,2,min)
for(i in 1:length(slicerange)){
  foo <- xst.aicc[[i]] - min.aicc[i]
  xstdelaic[,i] <- as.numeric(foo)
}
min(xstdelaic[xstdelaic !=0])  # 7.560771

# row.names(xst.aicc) <- modlist
# rowMeans(xst.aicc)
# BM        OU        EB        delta 
# -251.1975 -278.8923 -249.1171 -268.8743 

#matplot(t(xst.Sig2),type="l", lty=1, xlab="XST")


# Use regression discontinuity analysis to demonstrate that the handwing and armwing 
# have different amounts of disparity and different evolutionary rates
# First camber
foo <- rdd_data((camb.disparity.l$disparity[[1]]$elements), slicerange, cutpoint = 25)
camb.rdd <- rdd_reg_lm(rdd_object = foo, slope = "separate")

foo <- rdd_data(as.numeric(camb.Sig2[1,]), slicerange, cutpoint = 25)
camb.rdd.sig2 <- rdd_reg_lm(rdd_object = foo, slope = "separate")

# Now chord
foo <- rdd_data((chord.disparity.l$disparity[[1]]$elements), slicerange, cutpoint = 25)
chord.rdd <- rdd_reg_lm(rdd_object = foo, slope = "separate")

foo <- rdd_data(as.numeric(chord.Sig2[1,]), slicerange, cutpoint = 25)
chord.rdd.sig2 <- rdd_reg_lm(rdd_object = foo, slope = "separate")

# Now xst
foo <- rdd_data((xst.disparity.l$disparity[[1]]$elements), slicerange, cutpoint = 25)
xst.rdd <- rdd_reg_lm(rdd_object = foo, slope = "separate")

foo <- rdd_data(as.numeric(xst.Sig2[1,]), slicerange, cutpoint = 25)
xst.rdd.sig2 <- rdd_reg_lm(rdd_object = foo, slope = "separate")

# Finally xsa
foo <- rdd_data((xsa.disparity.l$disparity[[1]]$elements), slicerange, cutpoint = 25)
xsa.rdd <- rdd_reg_lm(rdd_object = foo, slope = "separate")

foo <- rdd_data(as.numeric(xsa.Sig2[1,]), slicerange, cutpoint = 25)
xsa.rdd.sig2 <- rdd_reg_lm(rdd_object = foo, slope = "separate")

## Finally, let's calculate phylogenetic signal ---------------------------------
camber.sig <- data.frame()
for(i in 1:length(camber.summary.log[1,1:35])){
  foo <- phylosig(smalltree, camber.summary.log[,i],method="K", test=TRUE, nsim=1000)
  camber.sig[1,i] <- foo$K
  camber.sig[2,i] <- foo$P
  foo <- phylosig(smalltree, camber.summary.log[,i], method="lambda", test=TRUE, nsim=1000)
  camber.sig[3,i] <- foo$lambda
  camber.sig[4,i] <- foo$P
}

chord.sig <- data.frame()
for(i in 1:length(chord.summary.log[1,1:35])){
  foo <- phylosig(smalltree, chord.summary.log[,i],method="K", test=TRUE, nsim=1000)
  chord.sig[1,i] <- foo$K
  chord.sig[2,i] <- foo$P
  foo <- phylosig(smalltree, chord.summary.log[,i], method="lambda", test=TRUE, nsim=1000)
  chord.sig[3,i] <- foo$lambda
  chord.sig[4,i] <- foo$P
}

xst.sig <- data.frame()
for(i in 1:length(xst.summary.log[1,1:35])){
  foo <- phylosig(smalltree, xst.summary.log[,i],method="K", test=TRUE, nsim=1000)
  xst.sig[1,i] <- foo$K
  xst.sig[2,i] <- foo$P
  foo <- phylosig(smalltree, xst.summary.log[,i], method="lambda", test=TRUE, nsim=1000)
  xst.sig[3,i] <- foo$lambda
  xst.sig[4,i] <- foo$P
}

xsa.sig <- data.frame()
for(i in 1:length(xsa.summary.log[1,1:35])){
  foo <- phylosig(smalltree, xsa.summary.log[,i],method="K", test=TRUE, nsim=1000)
  xsa.sig[1,i] <- foo$K
  xsa.sig[2,i] <- foo$P
  foo <- phylosig(smalltree, xsa.summary.log[,i], method="lambda", test=TRUE, nsim=1000)
  xsa.sig[3,i] <- foo$lambda
  xsa.sig[4,i] <- foo$P
}

# Calculate means of phylogenetic signal
mean(as.numeric(camber.sig[1,1:24])) # 0.2519742 - HW camber K
mean(as.numeric(camber.sig[3,1:24])) # 0.6900834 - HW camber lambda

mean(as.numeric(chord.sig[1,1:24])) # 0.3167447 - HW chord K
mean(as.numeric(chord.sig[3,1:24])) # 0.8617863 - HW chord lambda

mean(as.numeric(xst.sig[1,1:24])) # 0.1713869 - HW xst K
mean(as.numeric(xst.sig[3,1:24])) # 0.6359979 - HW xst lambda

mean(as.numeric(xsa.sig[1,1:24])) # 0.1914997 - HW xsa K
mean(as.numeric(xsa.sig[3,1:24])) # 0.7938477 - HW xsa lambda

mean(as.numeric(camber.sig[1,26:35])) # 0.2829812 - AW camber K
mean(as.numeric(camber.sig[3,26:35])) # 0.7077952 - AW camber lambda

mean(as.numeric(chord.sig[1,26:35])) # 0.4102009 - AW chord K
mean(as.numeric(chord.sig[3,26:35])) # 0.8968299 - AW chord lambda

mean(as.numeric(xst.sig[1,26:35])) # 0.1609044 - AW xst K
mean(as.numeric(xst.sig[3,26:35])) # 0.6725216 - AW xst lambda

mean(as.numeric(xsa.sig[1,26:35])) # 0.2059661 - AW xsa K
mean(as.numeric(xsa.sig[3,26:35])) # 0.7598084 - AW xsa lambda

## Make some figures to depict the modularity results --------------------------
source("Data and Scripts/slicewiseProfiles_fig.R") # Pane a. of Fig. 2 for the manuscript
source("Data and Scripts/Fig-SlicewiseK.R")        # Pane b. for Fig. 2
source("Data and Scripts/modTest_hist_fig.R")      # Fig. 3
source("Data and Scripts/disparity_fig.R")         # Fig. 4
#source("Rcode/CRvsPartitions_fig.R")   # Not currently being used

# Does an OU model give different results? - Nope
# camb.Sig2.ou <- fitContinuous(smalltree, camber.summary, model= "OU")
# chord.Sig2.ou <- fitContinuous(smalltree, chord.summary.nd, model= "OU")
# xsa.Sig2.ou <- fitContinuous(smalltree, xsa.summary.nd, model= "OU")
# xst.Sig2.ou <- fitContinuous(smalltree, xst.summary.nd, model= "OU")

# camb.Sig2.w <- fitContinuous(smalltree, camber.summary, model= "EB")
# chord.Sig2.w <- fitContinuous(smalltree, chord.summary.nd, model= "EB")
# xsa.Sig2.w <- fitContinuous(smalltree, xsa.summary.nd, model= "EB")
# xst.Sig2.w <- fitContinuous(smalltree, xst.summary.nd, model= "EB")

## Functions -------------------------------------------------------------------

# Modularity test function
modularity.test<-function(A,partition.gp,iter=999){
  if (length(dim(A))==3){ x<-two.d.array(A)
  p<-dim(A)[1]; k<-dim(A)[2]
  if(length(partition.gp)!=p){stop("Not all landmarks are assigned to a partition.")}
  gps<-as.factor(rep(partition.gp,k,each = k, length=p*k))  }
  if (length(dim(A))==2){ x<-A
  if(length(partition.gp)!=ncol(x)){stop("Not all variables are assigned to a partition.")}
  gps<-as.factor(partition.gp)  }
  ngps<-nlevels(gps)
  CR<-function(x,gps){
    gp.names <- levels(gps)
    g<-factor(as.numeric(gps))
    ngps<-nlevels(g)
    S<-var(x)
    diag(S)<-0
    gps.combo <- combn(ngps, 2)
    CR.gp <- sapply(1:ncol(gps.combo), function(j){ # no loops
      S11<-S[which(g==gps.combo[1,j]),which(g==gps.combo[1,j])]
      S22<-S[which(g==gps.combo[2,j]),which(g==gps.combo[2,j])]
      S12<-S[which(g==gps.combo[1,j]),which(g==gps.combo[2,j])]
      sqrt(sum(colSums(S12^2))/sqrt(sum(S11^2)*sum(S22^2)))
    })
    if(length(CR.gp) > 1) CR.mat <- dist(matrix(0, length(CR.gp),)) else 
      CR.mat = 0
    for(i in 1:length(CR.mat)) CR.mat[[i]] <- CR.gp[i]
    CR.val <- mean(CR.gp) 
    return(CR.val)
  }
  if (length(dim(A))==2){
    CR.obs<-CR(x,gps)
    P.val<-1
    CR.val<-rep(0,iter)
    for(ii in 1:iter){
      partition.gp.r<-sample(partition.gp)
      gps.r<-as.factor(partition.gp.r)
      CR.r<-CR(x,gps.r)
      CR.val[ii]<-CR.r
      P.val<-ifelse(CR.r<=CR.obs, P.val+1,P.val) 
    }
    CR.val[iter+1]=CR.obs
    P.val<-P.val/(iter+1)
    hist(CR.val,30,freq=TRUE,col="gray",xlab="CR Coefficient",xlim=c(min(CR.val)-0.01,max(CR.val+0.01)))
    arrows(CR.obs,50,CR.obs,5,length=0.1,lwd=2)
    return(list(CR=CR.obs,pvalue=P.val))
  }
  if (length(dim(A))==3){
    CR.vals<-array(NA,dim=90)
    for (i in 0:89){
      rot.mat<-matrix(c(cos(i*pi/180),sin(i*pi/180),0,-sin(i*pi/180),cos(i*pi/180),
                        0,0,0,1),ncol=3)
      A.rot<-array(NA,dim=dim(A))
      for (j in 1:dim(A)[3]){
        A.rot[,,j]<-A[,,j]%*%rot.mat[1:k,1:k]
      }  
      x<-two.d.array(A.rot)
      CR.vals[i+1]<-CR(x,gps)
    } 
    CR.obs<-mean(CR.vals); CR.se<-sd(CR.vals)/90     
    i<-(which.min(abs(CR.vals - CR.obs))-1)  
    rot.mat<-matrix(c(cos(i*pi/180),sin(i*pi/180),0,-sin(i*pi/180),cos(i*pi/180),
                      0,0,0,1),ncol=3)
    A.rot<-array(NA,dim=dim(A))
    for (j in 1:dim(A)[3]){
      A.rot[,,j]<-A[,,j]%*%rot.mat[1:k,1:k]
    }  
    x<-two.d.array(A.rot)
    P.val<-1
    CR.val<-rep(0,iter)
    for(ii in 1:iter){
      partition.gp.r<-sample(partition.gp)
      gps.r<-as.factor(rep(partition.gp.r,k,each = k, length=p*k))
      CR.r<-CR(x,gps.r)
      CR.val[ii]<-CR.r
      P.val<-ifelse(CR.r<=CR.obs, P.val+1,P.val) 
    }
    CR.val[iter+1]=CR.obs
    P.val<-P.val/(iter+1)
    hist(CR.val,30,freq=TRUE,col="gray",xlab="CR Coefficient",xlim=c(0,100))
    arrows(CR.obs,50,CR.obs,5,length=0.1,lwd=2)
    return(list(CR=CR.obs,CInterval=CR.CI<-c(CR.obs-(CR.se*1.96),CR.obs+(CR.se*1.96)),pvalue=P.val))
  }
}